"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createBridge = void 0;
const node_events_1 = require("node:events");
const generic_pool_1 = __importDefault(require("generic-pool"));
const createBridge = (postgres) => {
    return class PostgresBridge {
        constructor(poolConfiguration) {
            var _a, _b;
            this.poolEvents = new node_events_1.EventEmitter();
            this.pool = generic_pool_1.default.createPool({
                create: async () => {
                    var _a, _b;
                    const connectionEvents = new node_events_1.EventEmitter();
                    const connection = postgres({
                        database: poolConfiguration.database,
                        host: (_a = poolConfiguration.host) !== null && _a !== void 0 ? _a : 'localhost',
                        idle_timeout: poolConfiguration.idleTimeoutMillis ? poolConfiguration.idleTimeoutMillis / 1000 : 0,
                        max: 1,
                        onnotice: (notice) => {
                            connectionEvents.emit('notice', {
                                code: notice.code,
                                file: notice.file,
                                line: notice.line,
                                message: notice.message,
                                routine: notice.routine,
                                severity: notice.severity,
                                where: notice.where,
                            });
                        },
                        password: poolConfiguration.password,
                        port: (_b = poolConfiguration.port) !== null && _b !== void 0 ? _b : 5432,
                        ssl: poolConfiguration.ssl,
                        username: poolConfiguration.user,
                    });
                    connection.events = connectionEvents;
                    return connection;
                },
                destroy: (client) => {
                    return client.end({
                        timeout: 5,
                    });
                },
            }, {
                max: (_a = poolConfiguration.max) !== null && _a !== void 0 ? _a : 10,
                min: (_b = poolConfiguration.min) !== null && _b !== void 0 ? _b : 0,
            });
        }
        async connect() {
            const connection = await this.pool.acquire();
            const compatibleConnection = {
                end: async () => {
                    await this.pool.destroy(connection);
                },
                off: connection.events.off.bind(connection.events),
                on: connection.events.on.bind(connection.events),
                query: async (sql, parameters) => {
                    var _a, _b;
                    // https://github.com/porsager/postgres#result-array
                    const resultArray = await connection.unsafe(sql, parameters);
                    return {
                        command: resultArray.command,
                        fields: (_b = (_a = resultArray.columns) === null || _a === void 0 ? void 0 : _a.map((column) => {
                            return {
                                dataTypeID: column.type,
                                name: column.name,
                            };
                        })) !== null && _b !== void 0 ? _b : [],
                        rowCount: resultArray.count,
                        rows: Array.from(resultArray),
                    };
                },
                release: async () => {
                    await this.pool.release(connection);
                },
            };
            this.poolEvents.emit('connect', compatibleConnection);
            return compatibleConnection;
        }
        _pulseQueue() {
            // TODO implement logic equivalent to https://github.com/brianc/node-postgres/blob/master/packages/pg-pool/index.js#L109-L152
        }
        _remove() {
            // TODO implement logic equivalent to https://github.com/brianc/node-postgres/blob/master/packages/pg-pool/index.js#L154-L164
        }
        get idleCount() {
            return this.pool.available;
        }
        off(eventName, listener) {
            return this.poolEvents.off(eventName, listener);
        }
        on(eventName, listener) {
            return this.poolEvents.on(eventName, listener);
        }
        get totalCount() {
            return this.pool.size;
        }
        get waitingCount() {
            return this.pool.pending;
        }
    };
};
exports.createBridge = createBridge;
